package credit_usage

import (
	"context"
	"github.com/suyuan32/simple-admin-core/rpc/types/core"
	"time"

	"wechat-api/ent/creditusage"
	"wechat-api/ent/predicate"
	"wechat-api/internal/svc"
	"wechat-api/internal/types"
	"wechat-api/internal/utils/dberrorhandler"

	"github.com/suyuan32/simple-admin-common/msg/errormsg"

	"github.com/suyuan32/simple-admin-common/utils/pointy"
	"github.com/zeromicro/go-zero/core/logx"
)

type GetCreditUsageListLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetCreditUsageListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetCreditUsageListLogic {
	return &GetCreditUsageListLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GetCreditUsageListLogic) GetCreditUsageList(req *types.CreditUsageListReq) (*types.CreditUsageListResp, error) {
	isAdmin := l.ctx.Value("isAdmin").(bool)
	var predicates []predicate.CreditUsage
	if isAdmin {
		if req.OrganizationId != nil {
			predicates = append(predicates, creditusage.OrganizationID(*req.OrganizationId))
		}
	} else {
		organizationId := l.ctx.Value("organizationId").(uint64)
		predicates = append(predicates, creditusage.OrganizationID(organizationId))
	}

	if req.Ntype != nil && *req.Ntype > 0 {
		predicates = append(predicates, creditusage.Ntype(*req.Ntype))
	}

	// 这里是为限制时间段
	if req.StartDate != nil && *req.StartDate != "" {
		startDate, err := time.Parse("2006-01-02 15:04:05", *req.StartDate)
		if err != nil {
			l.Logger.Errorf("时间字符串startDate转换错误: %v", err)
		} else {
			predicates = append(predicates, creditusage.CreatedAtGTE(startDate))
		}
	}
	if req.EndDate != nil && *req.EndDate != "" {
		endDate, err := time.Parse("2006-01-02 15:04:05", *req.EndDate)
		if err != nil {
			l.Logger.Errorf("时间字符串endDate转换错误: %v", err)
		} else {
			predicates = append(predicates, creditusage.CreatedAtLTE(endDate))
		}
	}

	data, err := l.svcCtx.DB.CreditUsage.Query().Where(predicates...).Page(l.ctx, req.Page, req.PageSize)

	if err != nil {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}

	resp := &types.CreditUsageListResp{}
	resp.Msg = errormsg.Success
	resp.Data.Total = data.PageDetails.Total

	departmentInfo, err := l.svcCtx.CoreRpc.GetDepartmentById(l.ctx, &core.IDReq{Id: *req.OrganizationId})
	if err != nil {
		return nil, err
	}

	for _, v := range data.List {
		resp.Data.Data = append(resp.Data.Data,
			types.CreditUsageInfo{
				BaseIDInfo: types.BaseIDInfo{
					Id:        &v.ID,
					CreatedAt: pointy.GetPointer(v.CreatedAt.UnixMilli()),
					UpdatedAt: pointy.GetPointer(v.UpdatedAt.UnixMilli()),
				},
				Number:           &v.Number,
				BeforeNumber:     &v.BeforeNumber,
				AfterNumber:      &v.AfterNumber,
				PayNumber:        &v.PayNumber,
				PayMethod:        &v.PayMethod,
				Status:           &v.Status,
				Ntype:            &v.Ntype,
				Table:            &v.Table,
				OrganizationId:   &v.OrganizationID,
				OrganizationName: departmentInfo.Name,
				Nid:              &v.Nid,
				Reason:           &v.Reason,
				Operator:         &v.Operator,
			})
	}

	return resp, nil
}
